/**********************************************************************************************/
/* FBL_Sharable.h																			  */
/*                                                                       					  */
/* Foundation Base Library ( FBL )		                                   					  */
/*                                                                       					  */
/* Copyright Paradigma, 1993-2002                                        					  */
/* All Rights Reserved.                                                 					  */
/**********************************************************************************************/
//
//	Classes for reference counted objects which delete themselves when
//	no longer used.
//
//	Sharable objects must be allocated via "new".
//
//	Use FBL_Sharable as a mixin class for objects that you wish to share
//	and have deleted when they are no longer referenced.
//
//	Use FBL_StSharable for a stack-based object which maintains a reference to
//	a shared object for the duration of single function or statement block.
//
//	Use FBL_TSharablePtr template class as a member variable in a class where
//	you want to retain a reference to a shared object for the lifetime
//	of the user object.

#ifndef __FBL_Sharable_h 
	#define __FBL_Sharable_h

#if VSDK_FRAMEWORK_STYLE
#	include <VSDK/FBL.h>
#else
#	include <FBL.h>
#endif // VSDK_FRAMEWORK_STYLE


/**********************************************************************************************/
//	Mixin class for reference counted objects

class FBL_Sharable 
{
	public://///////////////////////////////////////////////////////////////////////////////////
	 
								FBL_Sharable( void ) { 
									mUseCount = 0; 
								}
							
								FBL_Sharable( const FBL_Sharable& /*inSharable*/ ) {
									mUseCount = 0;			// Copy has no users
								}

		FBL_Sharable&			operator = ( const FBL_Sharable& inSharable ) {
									if( this != &inSharable ) 
										mUseCount = 0;		// Copy has no users
									return *this;
								}

	 	void					AddUser( void ) {
									mUseCount++;
								}

 		void					RemoveUser( void ) {
									CHECK_FBL( mUseCount > 0 );	
									mUseCount--;

									if( mUseCount == 0 )
										delete this;
								}
					
		ulong					GetUseCount( void ) const { return mUseCount; }


	protected:////////////////////////////////////////////////////////////////////////////////// 
						
								// protected destructor prevents stack-based objects
virtual							~FBL_Sharable( void ){}	


	protected:////////////////////////////////////////////////////////////////////////////////// 

		ulong					mUseCount;
};


/**********************************************************************************************/
//	Template class for automatically adding and removing a user of a FBL_Sharable object
//
//	Typically, you will use a FBL_TSharablePtr as a member variable of a class. 
//  The coercion operator T* and the overloaded operator-> 
// let you use a FBL_TSharablePtr as if it were a T* variable.

template<class T>		// T must be subclass of FBL_Sharable
class FBL_TSharablePtr 
{
	public://///////////////////////////////////////////////////////////////////////////////////
		 
								FBL_TSharablePtr( T* inSharablePtr = NULL ) {
									mSharablePtr = NULL;
									SetReference(inSharablePtr);
								}

								FBL_TSharablePtr( const FBL_TSharablePtr<T>& inSharableRef ) {
									mSharablePtr = NULL;
									SetReference(inSharableRef.mSharablePtr);
								}

								~FBL_TSharablePtr( void ) { 
									SetReference(NULL);
								}

								operator T*( void ) const { 
									return mSharablePtr; 
								}

		FBL_TSharablePtr<T>& 	operator = ( T* inSharablePtr ) {
									SetReference(inSharablePtr);
									return *this;
								}

		FBL_TSharablePtr<T>&	operator = ( const FBL_TSharablePtr<T>& inSharableRef ) {
									SetReference( inSharableRef.mSharablePtr );
									return *this;
								}

		T*						operator -> ( void ) const	{ return mSharablePtr; }
		T*						GetObject( void ) const		{ return mSharablePtr; }


	protected:////////////////////////////////////////////////////////////////////////////////// 

		void					SetReference( T* inSharablePtr)
								{
									T*	oldSharablePtr = mSharablePtr;

									// Do add before remove in case inSharable == mSharable
									if( (mSharablePtr = inSharablePtr) ) 
										inSharablePtr->AddUser();	

									if( oldSharablePtr ) 
										oldSharablePtr->RemoveUser();
								}


	protected:////////////////////////////////////////////////////////////////////////////////// 

		T*						mSharablePtr;

};


/**********************************************************************************************/
#endif // __FBL_Sharable_h
